          (* +----------------------------------------------------+
             |  Trac de segment selon l'algorithme de Bresenham  |
             +----------------------------------------------------+ *)


#directory "..";;  (* adapter en fonction des circonstances *)

open Graphx;;
#load "graphx.cmo";;

                             (* +---------------+
                                |  Utilitaires  |
                                +---------------+ *)

let r = 20;;  (* dimension des pixels simuls *)

(* allume un gros point *)
let allume {x=x; y=y} =
  set_color blue;
  fill_circle {x=x*r; y=y*r} (r/2)
;;

(* trace ligne en coordonnes simules *)
let ligne {x=a; y=b} {x=c; y=d} =
  set_color red;
  set_line_width 3;
  moveto {x=a*r; y=b*r};
  lineto {x=c*r; y=d*r}
;;

(* trace une grille *)
let grille() =
  let sx = size_x() and sy = size_y() in
  set_color black;
  set_line_width 1;
  for i = 0 to sx/r do moveto {x=i*r; y=r}; lineto {x=i*r; y=sy} done;
  for i = 1 to sy/r do moveto {x=0; y=i*r}; lineto {x=sx; y=i*r} done
;;

(* coordonnes simules de la souris *)
let souris() =
  let m = mouse_pos() in
  {x=(m.x + r/2)/r; y=(m.y + r/2)/r}
;;

(* affiche les coordonnes d'un vecteur *)
let show_coords {x=x; y=y} =
  set_color black;
  moveto {x=10; y=0};
  draw_string (Printf.sprintf "dx = %d  dy = %d" x y);
;;

(* signe d'un entier *)
let signe x = if x < 0 then -1 else if x > 0 then 1 else 0;;


                       (* +---------------------------+
                          |  Algorithme de Bresenham  |
                          +---------------------------+ *)

(* trace le segment (a,b) (c,d) *)
let bresenham {x=a; y=b} {x=c; y=d} =

  (* Initialisation des incrments *)
  let l = abs(c - a)
  and h = abs(d - b) in
  let incrE  = 2*(min h l)
  and incrNE = -2*abs(h - l) in
  let d  = ref((incrE + incrNE)/2)
  and dx = signe(c - a)
  and dy = signe(d - b)
  and x = ref a
  and y = ref b
  in

  (* trac du premier point *)
  allume {x = !x; y = !y};

  (* |pente| <= 1 *)
  if l >= h then for i = 1 to l do
    x := !x + dx;
    if !d <= 0 then d := !d + incrE
               else begin d := !d + incrNE; y := !y + dy end;
    allume {x = !x; y = !y}
  done

  (* |pente| > 1 *)
  else for i = 1 to h do
    y := !y + dy;
    if !d <= 0 then d := !d + incrE
               else begin d := !d + incrNE; x := !x + dx end;
    allume {x = !x; y = !y}
  done

;;


                       (* +-----------------+
                          |  Dmonstration  |
                          +-----------------+ *)

type tmode = Termin | Point1 | Point2;;

begin

  print_string "trac interactif de segments ...\n";
  print_string "appuyer sur [e] pour effacer\n";
  print_string "appuyer sur [f] pour enregistrer au format FIG\n";
  print_string "appuyer sur [p] pour enregistrer au format postscript\n";
  print_string "appuyer sur [k] pour enregistrer au format pstricks\n";
  print_string "appuyer sur [g] pour enregistrer au format tga\n";
  print_string "appuyer sur [n] pour enregistrer au format ppm\n";
  print_string "appuyer sur [r] pour enregistrer au format ppmraw\n";
  print_string "appuyer sur [t] pour arrter\n";
  flush stdout;

  open_graph " 800x600";
  grille();

  let p0   = ref {x=0; y=0}
  and p1   = ref {x=0; y=0}
  and mode = ref Point1
  and img  = ref(get_image 0 0 (size_x()) (size_y())) 
  and vimg = ref(current_vimage())
  in

  while !mode <> Termin do
  match wait_next_event [Button_down; Button_up; Mouse_motion; Key_pressed] with

  | {keypressed=true; key='f'} -> write_fig        "bresenham.fig"
  | {keypressed=true; key='p'} -> write_postscript "bresenham.ps"
  | {keypressed=true; key='k'} -> write_pstricks   "bresenham.tex"
  | {keypressed=true; key='g'} -> write_tga        "bresenham.tga"
  | {keypressed=true; key='n'} -> write_ppm        "bresenham.ppm"
  | {keypressed=true; key='r'} -> write_ppmraw     "bresenham.ppm"

  | {keypressed=true; key='e'} -> clear_graph(); grille(); vimg := current_vimage()
  | {keypressed=true; key='t'} -> mode := Termin
  | {keypressed=true}          -> sound 440 100

  | {button=true} when !mode = Point1 ->
      p0   := souris();
      img  := get_image 0 0 (size_x()) (size_y());
      mode := Point2

  | {button=true} ->
      let m = souris() in
      if m <> !p1 then begin
        draw_image !img 0 0;
        bresenham !p0 m;
        ligne !p0 m;
        show_coords {x = m.x - !p0.x; y = m.y - !p0.y};
        p1 := m
      end

  | {button=false} when !mode = Point2 ->
      clear_graph();
      display_vimage !vimg;
      bresenham !p0 !p1;
      ligne !p0 !p1;
      vimg := current_vimage();
      mode := Point1


  | _ -> ()

  done;
  close_graph()

end;;
