(*****************************************************************************
 *                                                                           *
 *      Interface graphique : adaptation de la bibliothque libgraph         *
 *                                                                           *
 *****************************************************************************)

type point = Graph.point = { x: int; y: int };;
type color = Graph.color;;
type image = Graph.image;;

type graphic_cmd = Graph.graphic_cmd =
    | Set_color of color
    | Set_line_width of int
    | Set_font of string
    | Set_text_size of int
    | Poly of bool * point array
    | Bezier of bool * int * point array
    | Arc of bool * point * int * int * int * int
    | Ellipse of bool * point * int * int
    | String of point * string;;

type vimage = Graph.vimage;;

type status = Graph.status = {
 mouse: point;
 button: bool;
 keypressed: bool;
 key: char
 };;

type event = Graph.event =
 | Button_down
 | Button_up
 | Key_pressed
 | Mouse_motion
 | Poll;;

type bounding_box = Graph.bounding_box= | VIDE | BB of Graph.bb;;
type bb = Graph.bb = {xmin:int; xmax:int; ymin:int; ymax:int};;

let open_graph		= Graph.open_graph
and close_graph		= Graph.close_graph
and clear_graph		= Graph.clear_graph
and size_x		= Graph.size_x
and size_y		= Graph.size_y
and rgb			= Graph.rgb
and set_color		= Graph.set_color
and black		= Graph.black
and white		= Graph.white
and red			= Graph.red
and green		= Graph.green
and blue		= Graph.blue
and yellow		= Graph.yellow
and cyan		= Graph.cyan
and magenta		= Graph.magenta
and background		= Graph.background
and foreground		= Graph.foreground
and plot		= Graph.plot
and point_color		= Graph.point_color
and moveto		= Graph.moveto
and current_point	= Graph.current_point
and lineto		= Graph.lineto
and draw_arc		= Graph.draw_arc
and draw_ellipse	= Graph.draw_ellipse
and draw_circle		= Graph.draw_circle
and set_line_width	= Graph.set_line_width
and draw_char		= Graph.draw_char
and draw_string		= Graph.draw_string
and set_font		= Graph.set_font
and set_text_size	= Graph.set_text_size
and text_size		= Graph.text_size
and fill_rect		= Graph.fill_rect
and fill_poly		= Graph.fill_poly
and fill_arc		= Graph.fill_arc
and fill_ellipse	= Graph.fill_ellipse
and fill_circle		= Graph.fill_circle
and wait_next_event	= Graph.wait_next_event
and mouse_pos		= Graph.mouse_pos
and button_down		= Graph.button_down
and read_key		= Graph.read_key
and key_pressed		= Graph.key_pressed
and sound		= Graph.sound
and curveto		= Graph.curveto
and draw_poly		= Graph.draw_poly
and poly_of_bezier	= Graph.poly_of_bezier
and draw_bezier		= Graph.draw_bezier
and fill_bezier		= Graph.fill_bezier
and transp		= Graph.transp
and make_image		= Graph.make_image
and dump_image		= Graph.dump_image
and draw_image		= Graph.draw_image
and get_image		= Graph.get_image
and create_image	= Graph.create_image
and blit_image		= Graph.blit_image
and current_image	= Graph.current_image
and set_record		= Graph.set_record
and current_vimage	= Graph.current_vimage
and display_vimage	= Graph.display_vimage
and bounding_box	= Graph.bounding_box

and output_postscript	= Postscript.output_postscript
and output_fig		= Fig.output_fig   
and output_pstricks	= Pstricks.output_pstricks
and output_tga          = Tga.output_tga
and output_ppm          = Ppm.output_ppm
and output_ppmraw       = Ppm.output_ppmraw
;;

let write format img nom =
  let ch  = open_out nom in
  format ch (img());
  close_out ch
;;

let write_postscript    = write (output_postscript true) current_vimage
and write_fig           = write (output_fig        true) current_vimage
and write_pstricks      = write (output_pstricks   true) current_vimage
and write_tga           = write output_tga    current_image
and write_ppm           = write output_ppm    current_image
and write_ppmraw        = write output_ppmraw current_image
;;
